@@ -29,22 +29,22 @@ class xlator (Translator):
 
     def __init__ (self, c_this):
         self.requests = {}
-        Translator.__init__(self,c_this)
+        Translator.__init__(self, c_this)
 
     def lookup_fop (self, frame, this, loc, xdata):
         pargfid = uuid2str(loc.contents.pargfid)
         print("lookup FOP: %s:%s" % (pargfid, loc.contents.name))
         # Check the cache.
-        if cache.has_key(pargfid):
+        if pargfid in cache:
             if loc.contents.name in cache[pargfid]:
                 print("short-circuiting for %s:%s" % (pargfid,
                     loc.contents.name))
-                dl.unwind_lookup(frame,0,this,-1,2,None,None,None,None)
+                dl.unwind_lookup(frame, 0, this, -1, 2, None, None, None, None)
                 return 0
         key = dl.get_id(frame)
         self.requests[key] = (pargfid, loc.contents.name[:])
         # TBD: get real child xl from init, pass it here
-        dl.wind_lookup(frame,POINTER(xlator_t)(),loc,xdata)
+        dl.wind_lookup(frame, POINTER(xlator_t)(), loc, xdata)
         return 0
 
     def lookup_cbk (self, frame, cookie, this, op_ret, op_errno, inode, buf,
@@ -55,17 +55,17 @@ class xlator (Translator):
         # Update the cache.
         if op_ret == 0:
             print("found %s, removing from cache" % name)
-            if cache.has_key(pargfid):
+            if pargfid in cache:
                 cache[pargfid].discard(name)
         elif op_errno == 2:    # ENOENT
             print("failed to find %s, adding to cache" % name)
-            if cache.has_key(pargfid):
+            if pargfid in cache:
                 cache[pargfid].add(name)
             else:
-                cache[pargfid] = set([name])
+                cache[pargfid] = {name}
         del self.requests[key]
-        dl.unwind_lookup(frame,cookie,this,op_ret,op_errno,
-                         inode,buf,xdata,postparent)
+        dl.unwind_lookup(frame, cookie, this, op_ret, op_errno,
+                         inode, buf, xdata, postparent)
         return 0
 
     def create_fop (self, frame, this, loc, flags, mode, umask, fd, xdata):
@@ -74,7 +74,7 @@ class xlator (Translator):
         key = dl.get_id(frame)
         self.requests[key] = (pargfid, loc.contents.name[:])
         # TBD: get real child xl from init, pass it here
-        dl.wind_create(frame,POINTER(xlator_t)(),loc,flags,mode,umask,fd,xdata)
+        dl.wind_create(frame, POINTER(xlator_t)(), loc, flags, mode, umask, fd, xdata)
         return 0
 
     def create_cbk (self, frame, cookie, this, op_ret, op_errno, fd, inode,
@@ -85,9 +85,9 @@ class xlator (Translator):
         # Update the cache.
         if op_ret == 0:
             print("created %s, removing from cache" % name)
-            if cache.has_key(pargfid):
+            if pargfid in cache:
                 cache[pargfid].discard(name)
         del self.requests[key]
-        dl.unwind_create(frame,cookie,this,op_ret,op_errno,fd,inode,buf,
-                         preparent,postparent,xdata)
+        dl.unwind_create(frame, cookie, this, op_ret, op_errno, fd, inode, buf,
+                         preparent, postparent, xdata)
         return 0
